/* 	

		Function heatsched() of Arvert 4.x
		P. Zeitler, Lehigh University
		7/8/00
		
   Generates a heating schedule based on the losses observed for the goal spectrum.
   
		It's best to create model age spectra which match the goal spectrum, rather than
		use the actual lab heating schedule but end up with steps which don't match. In an ideal world, of course,
		the lab schedule would give you the same steps. But it's not an ideal world. That being so, 
		it's visually easier to compare results, and fitting is more straightforward if the goal and model spectra
		have the same step distribution.
		
		This does beg the question, though, about samples in which the modeled heating schedule
		doesn't match the observed one very well. Either something is wrong with your inputs (e.g., kinetic data),
		or the sample was not kinetically well behaved.
  	
*/

#include "arvert.h"

void heatsched(void)
{
	extern long geometry, goalsteps;
	extern double timelab[101], temperaturelab[101];
	extern double E[11],D0[11],fraction[11], goal_loss[101];
	extern long ndomains, labsteps;

	long step, i, j;
	double target[101],cumDta2[21],guess[4],Arloss[4];
	double Dta2, a_term, volume, delta;
//	extern char suffix[10];
	
//	long dummy;
	labsteps = goalsteps; // override what user might have entered in input file heatsched.in
	
	for (i = 1; i <= 10; i++)
	{
		cumDta2[i] = 0.0;
	}

	for (i = 1; i <= labsteps; i++)
	{
		target[i] = goal_loss[i];
		assert((target[i] <= 1.0) && (target[i] != target[i-1]));
		if (target[i] == 1.0) //   protect against case of total loss
		{
			target[i] = 0.9999;
		}
	}

	delta = 1.0;
	guess[1] = 10; // these guesses MUST bracket real temperature value for first step!! 
	guess[2] = 3000;
	do                    //  deal with first step
	{
		guess[3] = (guess[1] + guess[2])/2;
		for (i = 1; i <= 3; i++) 
		{
			Arloss[i] = 0.0;
			volume = 0;
			for (j = 1; j <= ndomains; j++)
			{
				volume = volume + fraction[j];
				a_term = D0[j]/31557600e6 * 600;  // Do/a2 back to 1/sec, then times 10 min to get Dot/a2
				cumDta2[j] = a_term * exp(-E[j] / R / (guess[i] + 273.15));
				if (geometry == 1)
				{
					if (cumDta2[j] <= 0.000001)
					{
						Arloss[i] = Arloss[i] + fraction[j]*(6 * sqrt(cumDta2[j]/pi));
					}
					else
					{
						long index;
						double sumseries, previous, change;
						sumseries = 0.0;
						previous = 1.0;
						index = 0;
						do
						{
							index = index + 1;
							change = fabs(previous - sumseries)/previous;
							previous = sumseries;
							sumseries = sumseries + 6/pi/pi/index/index*exp(-index*index*pi*pi*cumDta2[j]);
						}
						while (change > 0.000000001);
						Arloss[i] = Arloss[i] + fraction[j]*(1 - sumseries);
					}
				}
				else
				{
					if (cumDta2[j] < 0.159)
					{
						Arloss[i] = Arloss[i] + fraction[j]*(1.12838 * sqrt(cumDta2[j]));
					}
					else
					{
						Arloss[i] = Arloss[i] + fraction[j]*(1 - 0.810569 * exp(-2.467401 * cumDta2[j]));
					}
				}  // end if-geometry
			}   // end for-j to domains					
			
			Arloss[i] = Arloss[i]/volume;

		} // for loop calculating all guess temperatures 
		
		if (Arloss[3] <= target[1])
		{
			guess[1] = guess[3];
		}
		else
		{
			guess[2] = guess[3];
		}
		delta = fabs((Arloss[3] - target[1]))/target[1];
	}
	while (delta > 0.001);  // end conditional loop iterating guess temperatures 
	
	temperaturelab[1] = guess[3] + 273.15;
	
	for (j = 1; j <= ndomains; j++)
	{
		a_term = D0[j]/31557600e6 * 600;   // Dt/a2 for ten minutes
		cumDta2[j] = a_term * exp(-E[j] / R / (guess[3] + 273.15));
	}
		
	for (step = 2; step <= labsteps; step++)   /// now deal with all other steps
	{
		delta = 1.0;
		guess[1] = 10;  // for the record, guess temps work in C
		guess[2] = 3000;
		do
		{
			guess[3] = (guess[1] + guess[2])/2;
			for (i = 1; i <= 3; i++)
			{
				Arloss[i] = 0.0;
				for (j = 1; j <= ndomains; j++)
				{
					a_term = D0[j]/31557600e6 * 600;
					Dta2 = cumDta2[j] + a_term * exp(-E[j] / R / (guess[i] + 273.15));
//	printf("step %d   domain %d   Dta2 %e\n",step,j,Dta2);
				if (geometry == 1)
				{
					if (Dta2 <= 0.000001)
					{
						Arloss[i] = Arloss[i] + fraction[j]*(6 * sqrt(Dta2/pi));
					}
					else
					{
						long index;
						double sumseries, previous, change;
						sumseries = 0.0;
						previous = 1.0;
						index = 0;
						do
						{
							index = index + 1;
							change = fabs(previous - sumseries)/previous;
							previous = sumseries;
							sumseries = sumseries + 6/pi/pi/index/index*exp(-index*index*pi*pi*Dta2);
						}
						while (change > 0.000000001);
						Arloss[i] = Arloss[i] + fraction[j]*(1 - sumseries);
					}
				}
				else
					{
						if (Dta2 < 0.159)
						{
							Arloss[i] = Arloss[i] + fraction[j]*(1.12838 * sqrt(Dta2));
						}
						else
						{
							Arloss[i] = Arloss[i] + fraction[j]*(1 - 0.810569 * exp(-2.46740 * Dta2));
						}
					}
				}
				Arloss[i] = Arloss[i]/volume;
			} //  loop through guess temperatures
			
// printf("step %d Arloss[1 - 3] is %f   %f   %f\n",step, Arloss[3],target[step],guess[3]);
			if (Arloss[3] <= target[step])
			{
				guess[1] = guess[3];
			}
				else
			{
				guess[2] = guess[3];
			}
			delta = fabs(Arloss[3] - target[step])/target[step];
		}
		while (delta > 0.001);
	
		temperaturelab[step] = guess[3] + 273.15;  // convert degrees C to K
		for (j = 1; j <= ndomains; j++)
		{
			a_term = D0[j]/31557600e6 * 600;   // Dt/a2 for ten minutes
			cumDta2[j] = cumDta2[j] + a_term * exp(-E[j] / R / (guess[3] + 273.15));
		}
	}  

	for (i = 1; i <= labsteps; i++)
	{
		timelab[i] = 10/5.256e11;  // minutes --> m.y.
	}
	
return;
}