/* 	

		Functions selectsubset() and randomselect() of Arvert 4.x
		P. Zeitler, Lehigh University
		8/6/00
		
		This is THE key routine for constrained-random-search algorithm. It creates a new
		thermal history using the subset of thermal histories plus one additional thermal
		history. The new thermal history is create by projecting the temperature values of
		the one selected history through the centroid of the subset of histories; this projection
		is amplified by an amount specified by the input variable <amplify>, which typically
		has a value of about 1.3 (values much lower don't sufficiently explore parameter
		space, and values much larger slow convergence). Example: if amplify is 1.3, the
		temperature at a time node is 100, and the centroid (mean) value for the pool is
		130 at the same time node, then the temperature for the new CRS history at this time
		mode will be (130 - 100)* 1.3 + 130, or 169.
		
		Note (as pointed out by Sean Willet in his CRS code for fission-track annealing), that
		it might make sense to use a weighted mean in creating the centroid for the subset, the
		weighting being based on the exponential temperature dependence of diffusion. At present,
		however, the code below use just an arithmetic mean to create the centroid values.
		
		This routine works in four parts:
		
		(1) determines centroid values for subset
		(2) makes new CRS history

	  	(3) checks to make sure this history is reasonable
	  		(a) repairs any temperatures that have sneaked across explicit bounds
	  		(b) makes no attempt to repair violations of implicit constraints -- history is rejected
	  	
	  	
		(4) loads this new history into array temperaturein[nn] for use in lovera()
		
		3/15/03 -- narrowed explicit constraints
		
		3/30/03 -- restored implicit rate constraints. No repair attempt if history illegal--just retry whole process.

*/

#include "arvert.h"

// ------- Function newhistoryCRS() -------------------------

void newhistoryCRS(double **poolTemp)
{
	extern double amplify, temperaturein[nodes], timein[nodes];
	extern double upperlimit[nodes], lowerlimit[nodes];
	extern long CRS_selection, subsetsize, CRSselections[302];
	extern long ttpoints, goodHistory;
	extern double coolRate, heatRate;

	long i, j, index;
	double centroid[nodes],rate;

// first, determine the centroid values for the subset

	for (i = 1; i <= ttpoints; i++)  // zero centroid[12] array
	{
		centroid[i] = 0.0;
	}

	for (i = 1; i <= ttpoints; i++) // loop through the selected histories to get sum at each time node
	{
		for (j = 1; j <= subsetsize; j++)
		{
			index = CRSselections[j];
			centroid[i] = centroid[i] + poolTemp[index][i];
		}
	}
	
	for  (i = 1; i <= ttpoints; i++)   // get mean at each time node
	{
		centroid[i] = centroid[i] / subsetsize;
	}

// now, actually build the new CRS history
	for  (i = 1; i <= ttpoints; i++) // loop through the time nodes
	{
		temperaturein[i] = centroid[i] + amplify * (centroid[i] - poolTemp[CRS_selection][i]);
	}

// next, check the new history for points that exceed explicit bounds
// Deal with these extreme points by setting them just inside the explicit limits (old contraction code below)
/*
	for  (i = 1; i <= ttpoints; i++) // loop through the time nodes
	{
		printf("%d  %lf  %lf  %lf\n",i, temperaturein[i],upperlimit[i],lowerlimit[i]);
		while ((temperaturein[i] > upperlimit[i]) || (temperaturein[i] < lowerlimit[i]))
		{
			temperaturein[i] = centroid[i] + 0.95*(temperaturein[i] - centroid[i]);
		}
	}
*/
	for  (i = 1; i <= ttpoints; i++) // loop through the time nodes
	{
		if (temperaturein[i] > upperlimit[i])
		{
			temperaturein[i]= upperlimit[i] - 1.0;
		}
		if (temperaturein[i] < lowerlimit[i])
		{
			temperaturein[i]= lowerlimit[i] + 1.0;
		}
	}
	
//printf("PAUSED for input\n");
//printf("  QUIT to abort model  --> ");
//scanf("%d", &i);
	
// now, check for violations of implicit rate constraints, and either try to fix them or exit routine in failure and retry from scratch

	goodHistory = 1;

	for  (i = 1; i <= ttpoints - 1; i++) // loop through the time nodes
	{
		rate = (temperaturein[i] - temperaturein[i+1])/(timein[i] - timein[i+1]);   // positive is cooling
	/*	
		j = 0;
		while (((rate > coolRate) || (abs(rate) > heatRate)) && (j < 0))  // try to repair one with 50% contraction of point 
		{
			j++;
			temperaturein[i+1] = centroid[i+1] + 0.50*(temperaturein[i+1] - centroid[i+1]);
			rate = (temperaturein[i] - temperaturein[i+1])/(timein[i] - timein[i+1]);
		}
		
	*/
		if ((rate >= 0.0) && (rate > coolRate))
		{
			goodHistory = -1;
		}	
		if ((rate < 0.0) && (abs(rate) > heatRate))
		{
			goodHistory = -1;
		}
	}


	return;
}